<?php

namespace Drupal\Tests\codemirror_editor\FunctionalJavascript;

/**
 * Tests the CodeMirror field widget.
 *
 * @group codemirror_editor
 */
class WidgetTest extends TestBase {

  /**
   * {@inheritdoc}
   */
  public static $modules = [
    'node',
    'field_ui',
  ];

  protected $widgetSettings = [];

  /**
   * Test callback.
   */
  public function testWidgetEditor() {

    $permissions = [
      'administer node fields',
      'administer node form display',
      'create test content',
      'edit any test content',
    ];
    $user = $this->drupalCreateUser($permissions);
    $this->drupalLogin($user);

    // Default widget settings.
    $this->widgetSettings = [
      'rows' => 5,
      'placeholder' => '',
      'toolbar' => TRUE,
      'lineNumbers' => FALSE,
      'foldGutter' => FALSE,
      'autoCloseTags' => TRUE,
      'styleActiveLine' => FALSE,
    ];

    $this->drupalGet('node/add/test');
    $this->assertForm();

    $this->drupalGet('admin/structure/types/manage/test/form-display');

    $this->assertSettingsSummary();

    $this->click('//input[@name = "field_description_settings_edit"]');

    $this->assertSettingsForm();

    $this->widgetSettings = [
      'rows' => 10,
      'placeholder' => 'Example',
      'toolbar' => FALSE,
      'lineNumbers' => TRUE,
      'foldGutter' => TRUE,
      'autoCloseTags' => FALSE,
      'styleActiveLine' => TRUE,
    ];

    $this->updateSettingField('rows');
    $this->updateSettingField('placeholder');
    $this->updateSettingField('toolbar');
    $this->updateSettingField('lineNumbers');
    $this->updateSettingField('foldGutter');
    $this->updateSettingField('autoCloseTags');
    $this->updateSettingField('styleActiveLine');

    $page = $this->getSession()->getPage();
    $page->pressButton('field_description_plugin_settings_update');

    $this->assertSession()
      ->waitForElementVisible('xpath', '//select[@name = "fields[field_description][type]"]');

    $page->pressButton('Save');
    $this->assertSession()->pageTextContains('Your settings have been saved.');
    $this->assertSettingsSummary();

    $this->drupalGet('node/add/test');
    $this->assertForm();
    $page->fillField('Title', 'Example');
    $this->editorSetValue('It works!');
    $page->pressButton('Save');
    $this->assertSession()->pageTextContains('It works!');

    $this->drupalGet('node/1/edit');
    $this->assertEditorValue('It works!');
  }

  /**
   * Asserts widget form.
   */
  protected function assertForm() {
    $xpath = sprintf(
      '//textarea[@name = "field_description[0][value]" and @rows = %d and @placeholder = "%s"]',
      $this->widgetSettings['rows'],
      $this->widgetSettings['placeholder']
    );
    $this->assertSession()->elementExists('xpath', $xpath);
    $this->widgetSettings['toolbar'] ? $this->assertToolbarExists() : $this->assertToolbarNotExists();
    $this->assertEditorOption('lineNumbers', $this->widgetSettings['lineNumbers']);
    $this->assertEditorOption('foldGutter', $this->widgetSettings['foldGutter']);
    $this->assertEditorOption('autoCloseTags', $this->widgetSettings['autoCloseTags']);
    $this->assertEditorOption('styleActiveLine', $this->widgetSettings['styleActiveLine']);
  }

  /**
   * Asserts widget settings summary.
   */
  protected function assertSettingsSummary() {
    $expected_summary[] = 'Number of rows: ' . $this->widgetSettings['rows'];
    if ($this->widgetSettings['placeholder']) {
      $expected_summary[] = 'Placeholder: ' . $this->widgetSettings['placeholder'];
    }
    $expected_summary[] = 'Load toolbar: ' . ($this->widgetSettings['toolbar'] ? 'Yes' : 'No');
    $expected_summary[] = 'Line numbers: ' . ($this->widgetSettings['lineNumbers'] ? 'Yes' : 'No');
    $expected_summary[] = 'Fold gutter: ' . ($this->widgetSettings['foldGutter'] ? 'Yes' : 'No');
    $expected_summary[] = 'Auto close tags: ' . ($this->widgetSettings['autoCloseTags'] ? 'Yes' : 'No');
    $expected_summary[] = 'Style active line: ' . ($this->widgetSettings['styleActiveLine'] ? 'Yes' : 'No');

    $summary_xpath = '//tr[@id = "field-description"]//div[@class = "field-plugin-summary"]';
    $summary = $this->xpath($summary_xpath)[0]->getHtml();

    self::assertEquals(implode('<br>', $expected_summary), $summary);
  }

  /**
   * Asserts widget settings form.
   */
  protected function assertSettingsForm() {
    $assert_session = $this->assertSession();

    $settings_wrapper = $assert_session
      ->waitForElementVisible('xpath', '//div[@data-drupal-selector = "edit-fields-field-description-settings-edit-form"]');

    $xpath = '//input[@name = "fields[field_description][settings_edit_form][settings][rows]" and @value = %d]';
    $xpath = sprintf($xpath, $this->widgetSettings['rows']);
    $assert_session->elementExists('xpath', $xpath, $settings_wrapper);

    $xpath = '//input[@name = "fields[field_description][settings_edit_form][settings][placeholder]" and @value = "%s"]';
    $xpath = sprintf($xpath, $this->widgetSettings['placeholder']);
    $assert_session->elementExists('xpath', $xpath, $settings_wrapper);

    $xpath = '//input[@name = "fields[field_description][settings_edit_form][settings][toolbar]" and %s]';
    $xpath = sprintf($xpath, $this->widgetSettings['toolbar'] ? '@checked = "checked"' : 'not(@checked)');
    $assert_session->elementExists('xpath', $xpath, $settings_wrapper);

    $xpath = '//input[@name = "fields[field_description][settings_edit_form][settings][lineNumbers]" and %s]';
    $xpath = sprintf($xpath, $this->widgetSettings['lineNumbers'] ? '@checked = "checked"' : 'not(@checked)');
    $assert_session->elementExists('xpath', $xpath, $settings_wrapper);

    $xpath = '//input[@name = "fields[field_description][settings_edit_form][settings][autoCloseTags]" and %s]';
    $xpath = sprintf($xpath, $this->widgetSettings['autoCloseTags'] ? '@checked = "checked"' : 'not(@checked)');
    $assert_session->elementExists('xpath', $xpath, $settings_wrapper);

    $xpath = '//input[@name = "fields[field_description][settings_edit_form][settings][styleActiveLine]" and %s]';
    $xpath = sprintf($xpath, $this->widgetSettings['styleActiveLine'] ? '@checked = "checked"' : 'not(@checked)');
    $assert_session->elementExists('xpath', $xpath, $settings_wrapper);
  }

  /**
   * Sets a value for a given settings field.
   */
  protected function updateSettingField($name) {
    $page = $this->getSession()->getPage();
    $field_name = "fields[field_description][settings_edit_form][settings][$name]";
    $value = $this->widgetSettings[$name];
    if (is_bool($value)) {
      $value ? $page->checkField($field_name) : $page->uncheckField($field_name);
    }
    else {
      $page->fillField($field_name, $value);
    }
  }

  /**
   * {@inheritdoc}
   */
  protected function getWrapperSelector() {
    return '.js-form-item-field-description-0-value';
  }

}
